/**
 * <p>Starting from a given directory (or a file), traverse
 * the directory structure recursively. For each file or 
 * directory encountered, print the name.</p>
 *
 * @author Xiannong Meng
 * @date
 */
#include "read_dir.h"

int main(int argc,char *argv[])
{
  void process(char *name);   // function prototype

  if (argc != 2)
  {
    printf(" usage: %s dirname \n", argv[0]);
    exit(1);
  }
  else
  {
    process(argv[1]);       /* pass in the starting directory */
  }

  return 0;
}

/**
 * <p>
 * Recursively go through the directory structure
 * using breadth-first search.</p>
 *
 * @param root The root of the file system to be visited
 */
void process(char *root)
{
  int numOfFiles = 0;
  void Enq(char *name, que_t *q);
  void Deq(que_t *q);
  void Front(char *name,que_t q);
  enum boolean Empty(que_t q);

  void initq(que_t *q);
  que_t nameq;
  char dname[MAXL], cname[MAXL], buf[MAXL], prefix[MAXL];

  struct dirent *dp;
  DIR *dirp;
  struct stat statbuf;    // status buffer

  /* initialize the queue with the first file/dir */
    initq(&nameq);
    Enq(root,&nameq);

    while (Empty(nameq) != TRUE)  // visit each file
    {
      Front(dname,nameq); 
      Deq(&nameq);
      dirp = opendir(dname);
      if (dirp != NULL)     /* it is a directory */
      {
         printf("directory : %s\n",dname);
         strcpy(prefix,dname);
         strcat(prefix,"/");
         for (dp = readdir(dirp); dp != NULL; dp = readdir(dirp))
         {
            if ((strcmp(dp->d_name,"..") != 0) && (strcmp(dp->d_name,".") != 0))
            {   /* prevent from infinite loop */
              strcpy(cname,prefix);
              strcat(cname,dp->d_name);   /* concatenate the prefix */
	      if (lstat(cname,&statbuf) == 0)
	      {
		  if ((statbuf.st_mode & 0xF000) != S_IFLNK)  
		      // not a symbolic link
		      Enq(cname,&nameq);
		  else
		      cerr << "skipped s-link " << prefix << dp->d_name
			   << endl;
	      }
	    }
	 }
	 closedir (dirp);
      }
      else                  /* it is a regular file */
      {
	   cout << " processing file : " << dname << endl;
           numOfFiles ++;
      }  /* if (dirp != NULL) else */
    }  /* while */

    cout << " a total of " << numOfFiles << " files were counted\n";
}

/**
 * <p>
 * Initialize the queue.
 * </p>
 *
 * @param q The variable that holds the queue
 */
void initq(que_t *q)
{
  q->head = q->tail = NULL;
}

/**
 * <p>
 * Check to see if a queue is empty.
 * </p>
 *
 * @param q The queue to be checked
 * @return true if the queue is empty, false otherwise
 */
enum boolean Empty(que_t q)
{
   if (q.head == NULL)
     return TRUE;
   else
     return FALSE;
}

/**
 * <p>
 * Insert a file name into the queue.
 * </p>
 *
 * @param name The file name to be inserted
 * @param q The reference to the queue
 */
void Enq(char *name, que_t *q)
{
   item_t *temp;

   temp = (item_t *)malloc(sizeof(item_t));
   strcpy(temp->name,name);
   temp->next = NULL;

   if (Empty(*q) == TRUE)
   {
      q->head = temp;
      q->tail = temp;
   }
   else
   {
      q->tail->next = temp;
      q->tail = q->tail->next;
   }
}

/**
 * <p>
 * Remove an item from the queue.
 * </p>
 *
 * @param q Reference to the queue 
 */
void Deq(que_t *q)
{
   item_t *temp;

   if (Empty(*q) == TRUE)
   {
     printf(" error in dequeue \n");
     exit(1);
   }
   else
   {
     temp = q->head;
     q->head = q->head->next;
     free(temp);
   }
}

/**
 * <p>
 * Return the first element in the queue, if not empty.
 * </p>
 *
 * @param name  The reference item (file name) returned by the function
 * @param q     The queue from which the first item is removed
 */
void Front(char *name,que_t q)
{
  if (Empty(q) == TRUE)
   {
     printf(" error in dequeue \n");
     exit(1);
   }
   else
     strcpy(name, q.head->name);
} 

